/***************************************************************************
                          ApollonBrowseView.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
using namespace std;

#include <qstring.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qpopupmenu.h>
#include <qurl.h>
#include <qtooltip.h>

#include <kiconloader.h>
#include <kglobal.h>
#include <klocale.h>
#include <qregexp.h>  

#include "apollonbrowseview.h"
#include "apollonbrowseviewitem.h"
#include "giftconnection.h"

ApollonBrowseView::ApollonBrowseView( QWidget* parent, const int ID, const QString& text ):
                 QVBox( parent )
{
	_ID = ID;
	query = text;
	rootlistitem = NULL;

	KIconLoader *loader = KGlobal::iconLoader();

	QHBox* box = new QHBox(this);

	iconlabel = new QLabel(box);
	label = new QLabel(box);

	QString realm;

	iconlabel->setPixmap( loader->loadIcon("connect_established",KIcon::Small) );

	labeltext_search = i18n("Browsing user: <i>%1</i> - Searching...").arg(text);
	labeltext_complete = i18n("Browsing user: <i>%1</i> - Search completed!").arg(text);

	label->setText( labeltext_search );


	QPushButton* stop = new QPushButton(box);
	stop->setPixmap(loader->loadIcon("player_pause", KIcon::Small));
	stop->setFixedHeight( stop->sizeHint().height() );
	stop->setFixedWidth( stop->sizeHint().width() );
	connect(stop, SIGNAL(clicked()), this, SLOT(stopSearch()));
	QToolTip::add(stop, i18n("Stop search"));

	QPushButton* close = new QPushButton(box);
	close->setPixmap(loader->loadIcon("fileclose", KIcon::Small));
	close->setFixedHeight( close->sizeHint().height() );
	close->setFixedWidth( close->sizeHint().width() );
	QToolTip::add(close, i18n("Close tab"));

	box->setStretchFactor(label, 4);
	box->setStretchFactor(close, 0);

	connect(close, SIGNAL(clicked()), this, SLOT(deleteThis()) );

	browseresults = new QListView(this);
	browseresults->setSelectionMode(QListView::Extended);

	browseresults ->addColumn(i18n("Name"));

	browseresults ->addColumn(i18n("Size"));
	browseresults->setColumnWidth(1,80);
	browseresults->setColumnAlignment(1, Qt::AlignRight);

	browseresults->setAllColumnsShowFocus(TRUE);
	browseresults->setRootIsDecorated(TRUE);
	browseresults->setMinimumSize(300, 150);
	browseresults->setShowSortIndicator(TRUE);
	browseresults->setSorting(1,FALSE);


	connect( browseresults, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(itemDoubleClicked(QListViewItem *) ) );
	connect( browseresults, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)), this, SLOT(sltPopUp(QListViewItem*, const QPoint&, int)));

	popupmenu = new  QPopupMenu(browseresults);
	popupmenu->insertItem(loader->loadIcon("2downarrow", KIcon::Small), i18n("Download"), this, SLOT(sltDownload()));

}

ApollonBrowseView::~ApollonBrowseView()
{
}

void ApollonBrowseView::addItem( QString searchreply )
{
	searchreply = searchreply.stripWhiteSpace();

	QString recievedSearchID = giFTConnection::searchKey( "ITEM", searchreply );
	if (recievedSearchID.toInt() == _ID )
	{
		/* This is what a browse user reply look like:
		   ITEM(901) user(ronin@209.89.235.12) node(ronin@209.89.235.12) availability(0) size(6193164)
		   url(OpenFT://209.89.235.12:2740/mp3/Seb%20Fontaine/08%20-%20Moroccan%20Blonde%20-%20Release.mp3)
		   file(/mp3/Seb Fontaine/08 - Moroccan Blonde - Release.mp3) mime(audio/mpeg) hash(MD5:cce355225326ae2f3d983edd5b61ed28)
		   META { bitrate(160000) frequency(44100) duration(309) };
		*/
		if ( searchreply.find("file(") > -1 )
		{
			KIconLoader *loader = KGlobal::iconLoader();

			QUrl* url = new QUrl( giFTConnection::searchKey("url", searchreply) );

			if(!rootlistitem)
			{
				rootlistitem = new QListViewItem( browseresults, url->host() );
				rootlistitem->setPixmap(0, loader->loadIcon("connect_established", KIcon::Small));
				rootlistitem->setOpen(TRUE);
			}

			QString path = url->dirPath();
			QListViewItem* parent = getParent(path);

			ApollonBrowseViewItem* item = new ApollonBrowseViewItem( parent );

			QString temp  = giFTConnection::searchKey("file", searchreply);
			item->setFileName( temp.right(temp.length() - temp.findRev("/") - 1) );
			item->setUser( giFTConnection::searchKey("user", searchreply) );
			item->setSize( giFTConnection::searchKey("size", searchreply) );
			item->setUrl( giFTConnection::searchKey("url", searchreply) );
			item->setHash( giFTConnection::searchKey("hash", searchreply) );

		}
		else
		{
			QString endsearch = "ITEM(" + QString::number( _ID ) + ");";

			if ( QString::compare( endsearch, searchreply) == 0 )
			{
			        label->setText(labeltext_complete);
			}
			else
			{
			  //cout << searchreply << endl;
			}
		}
	}
}

void ApollonBrowseView::stopSearch()
{
	emit giftCancelBrowseRequest(_ID);
}


void ApollonBrowseView::deleteThis()
{
	// Make a call to the deamon not to search for thisone anymore
	// FIXME: make the call

	emit giftCancelBrowseRequest(_ID);

	delete this;
}

void ApollonBrowseView::downloadItemClicked(QListViewItem* item)
{
	ApollonBrowseViewItem* bvi = static_cast<ApollonBrowseViewItem*>(item);

	if( bvi->childCount() == 0 )
	{
		KIconLoader *loader = KGlobal::iconLoader();
		bvi->setPixmap(0, loader->loadIcon("2downarrow", KIcon::Small));

		emit giftAddRequest( bvi->getUser(), bvi->getHash(), bvi->getSize(), bvi->getUrl(), bvi->getFileName() );
	}
	else
	{
	item = item->firstChild();
	while(item)
	{
	downloadItemClicked(item);
	item = item->nextSibling();
	}
	}
}
void ApollonBrowseView::itemDoubleClicked(QListViewItem* item)
{
	ApollonBrowseViewItem* bvi = static_cast<ApollonBrowseViewItem*>(item);

	if( bvi->childCount() == 0 )
	{
		KIconLoader *loader = KGlobal::iconLoader();
		bvi->setPixmap(0, loader->loadIcon("2downarrow", KIcon::Small));

		emit giftAddRequest( bvi->getUser(), bvi->getHash(), bvi->getSize(), bvi->getUrl(), bvi->getFileName() );
	}
	else return;

}

QListViewItem*  ApollonBrowseView::getParent(QString dirpath)
{
	QListViewItem* parent = items[dirpath];
	if (!parent)
	{
		int index = dirpath.findRev('/');
		QString direlement = dirpath.mid(index + 1);
		QString parentdirpath = dirpath.left(index);

		if(parentdirpath == "")
		{
			QListViewItem* grandparent = rootlistitem;
			parent = new QListViewItem(grandparent, direlement);
			KIconLoader *loader = KGlobal::iconLoader();
			parent->setPixmap(0, loader->loadIcon("folder", KIcon::Small));
			items.insert(dirpath, parent);
		}

		if(parentdirpath != "")
		{
			QListViewItem* grandparent = getParent(parentdirpath);
			parent = new QListViewItem(grandparent, direlement);
			KIconLoader *loader = KGlobal::iconLoader();
			parent->setPixmap(0, loader->loadIcon("folder", KIcon::Small));

			items.insert(dirpath, parent);
		}
	}
	return parent;
}


void ApollonBrowseView::sltPopUp(QListViewItem* item, const QPoint & point, int col){
  popupmenu->popup(point);
}

void ApollonBrowseView::sltDownload(){
	QListViewItem* item = browseresults->firstChild();
	sltDownload(item);
}

void ApollonBrowseView::sltDownload(QListViewItem* item){
	QListViewItem* item1 = item;
	while (item1)
	{
	if (item1->isSelected())
	downloadItemClicked(item1);
	item1 = item1->nextSibling();
	}
	while (item)
	{
	QListViewItem* nextitem = item->firstChild();
	sltDownload(nextitem);
	item = item->nextSibling();
	}
}

